@@ -20,7 +20,7 @@
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Thomas Perl <m@thp.io>'
-__version__ = '0.6.1'
+__version__ = '0.6.2'
 __website__ = 'http://gpodder.org/podcastparser/'
 __license__ = 'ISC License'
 
@@ -48,7 +48,17 @@ except ImportError:
 
 try:
     # Python 2
-    from rfc822 import mktime_tz, parsedate_tz
+    from rfc822 import parsedate_tz
+    import calendar
+    # This is taken from Python 3's email._parseaddr, since it handles
+    # pre-epoch dates better than what Python 2 does (time.mktime())
+    def mktime_tz(data):
+        if data[9] is None:
+            # No zone info, so localtime is better assumption than GMT
+            return time.mktime(data[:8] + (-1,))
+        else:
+            t = calendar.timegm(data)
+            return t - data[9]
 except ImportError:
     # Python 3
     from email.utils import mktime_tz, parsedate_tz
@@ -517,6 +527,15 @@ def parse_pubdate(text):
     >>> parse_pubdate('Fri, 21 Nov 1997 09:55:06 -0600')
     880127706
 
+    >>> parse_pubdate('2003-12-13T00:00:00+02:00')
+    1071266400
+
+    >>> parse_pubdate('2003-12-13T18:30:02Z')
+    1071340202
+
+    >>> parse_pubdate('Mon, 02 May 1960 09:05:01 +0100')
+    -305049299
+
     >>> parse_pubdate('')
     0
 
@@ -528,13 +547,29 @@ def parse_pubdate(text):
 
     parsed = parsedate_tz(text)
     if parsed is not None:
-        return int(mktime_tz(parsed))
-
-    # TODO: Fully RFC 3339-compliant parsing (w/ timezone)
+        try:
+            pubtimeseconds = int(mktime_tz(parsed))
+            return pubtimeseconds
+        except(OverflowError,ValueError):
+            logger.warn('bad pubdate %s is before epoch or after end of time (2038)',parsed)
+            return 0
+        
     try:
         parsed = time.strptime(text[:19], '%Y-%m-%dT%H:%M:%S')
         if parsed is not None:
-            return int(time.mktime(parsed))
+            m = re.match(r'^(?:Z|([+-])([0-9]{2})[:]([0-9]{2}))$', text[19:])
+            if m:
+                parsed = list(iter(parsed))
+                if m.group(1):
+                    offset = 3600 * int(m.group(2)) + 60 * int(m.group(3))
+                    if m.group(1) == '-':
+                        offset = 0 - offset
+                else:
+                    offset = 0
+                parsed.append(offset)
+                return int(mktime_tz(tuple(parsed)))
+            else:
+                return int(time.mktime(parsed))
     except Exception:
         pass
 
@@ -560,6 +595,7 @@ MAPPING = {
     'rss/channel/item/link': EpisodeAttrRelativeLink('link'),
     'rss/channel/item/description': EpisodeAttr('description', squash_whitespace),
     'rss/channel/item/itunes:summary': EpisodeAttr('description', squash_whitespace),
+    'rss/channel/item/media:description': EpisodeAttr('description', squash_whitespace),
     'rss/channel/item/itunes:subtitle': EpisodeAttr('subtitle', squash_whitespace),
     'rss/channel/item/content:encoded': EpisodeAttr('description_html'),
     'rss/channel/item/itunes:duration': EpisodeAttr('total_time', parse_time),
@@ -582,11 +618,27 @@ MAPPING = {
     'atom:feed/atom:entry/atom:title': EpisodeAttr('title', squash_whitespace),
     'atom:feed/atom:entry/atom:link': AtomLink(),
     'atom:feed/atom:entry/atom:content': AtomContent(),
+    'atom:feed/atom:entry/content:encoded': EpisodeAttr('description_html'),
     'atom:feed/atom:entry/atom:published': EpisodeAttr('published', parse_pubdate),
+    'atom:feed/atom:entry/atom:updated': EpisodeAttr('published', parse_pubdate),
+    'atom:feed/atom:entry/media:group/media:description': EpisodeAttr('description', squash_whitespace),
     'atom:feed/atom:entry/psc:chapters': PodloveChapters(),
     'atom:feed/atom:entry/psc:chapters/psc:chapter': PodloveChapter(),
 }
 
+# Derive valid root elements from the supported MAPPINGs
+VALID_ROOTS = set(path.split('/')[0] for path in MAPPING.keys())
+
+
+class FeedParseError(sax.SAXParseException, ValueError):
+    """
+    Exception raised when asked to parse an invalid feed
+    
+    This exception allows users of this library to catch exceptions
+    without having to import the XML parsing library themselves.
+    """
+    pass
+
 
 class PodcastHandler(sax.handler.ContentHandler):
     def __init__(self, url, max_episodes):
@@ -635,6 +687,13 @@ class PodcastHandler(sax.handler.ContentHandler):
         if len(entry['chapters']) == 0:
             del entry['chapters']
 
+        # Ensures `description` does not contain HTML
+        if 'description' in entry and is_html(entry['description']):
+            if 'description_html' not in entry:
+                entry['description_html'] = entry['description']
+            entry['description'] = ''
+
+        # Sets `description` to stripped `description_html` when absent
         if 'description_html' in entry and not entry['description']:
             entry['description'] = remove_html_tags(entry['description_html'])
 
@@ -673,7 +732,14 @@ class PodcastHandler(sax.handler.ContentHandler):
 
     def startElement(self, name, attrs):
         self.namespace = Namespace(attrs, self.namespace)
-        self.path_stack.append(self.namespace.map(name))
+        name = self.namespace.map(name)
+        if not self.path_stack and name not in VALID_ROOTS:
+            raise FeedParseError(
+                msg='Unsupported feed type: {}'.format(name),
+                exception=None,
+                locator=self._locator,
+            )
+        self.path_stack.append(name)
 
         target = MAPPING.get('/'.join(self.path_stack))
         if target is not None:
@@ -707,7 +773,10 @@ def parse(url, stream, max_episodes=0):
     :returns: a dict with the parsed contents of the feed
     """
     handler = PodcastHandler(url, max_episodes)
-    sax.parse(stream, handler)
+    try:
+        sax.parse(stream, handler)
+    except sax.SAXParseException as e:
+        raise FeedParseError(e.getMessage(), e.getException(), e._locator)
     return handler.data
 
 
@@ -781,6 +850,12 @@ def normalize_feed_url(url):
     # urlunsplit might return "a slighty different, but equivalent URL"
     return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
+def is_html(text):
+    """
+    Tests whether the given string contains HTML encoded data
+    """
+    html_test = re.compile(r'<[a-z][\s\S]*>', re.IGNORECASE)
+    return bool(html_test.search(text))
 
 def remove_html_tags(html):
     """
